#ifndef INTERFACE_H
#define INTERFACE_H
#include <string>
#include <qstringlist.h>
using namespace std;
/**
 * @file interface.h
 * @brief nrc_lib 函数库
 * @author 纳博特科技 Inexbot
 * @date 2020-05-11
 * @version 2.0.0
 * @note 修改内容
 *   - 见二次开发文档
*/

/**
 * @brief 指令结构体
 */
typedef struct cmdPara
{
    /**
     * @brief 指令类型枚举
     *  - MOVJ 关节运动
     *  - MOVL 直线运动
     */
    enum Type{MOVJ=1, MOVL, MOVC, MOVS, MOVCA, MOVJEXT, MOVLEXT, MOVCEXT ,ARCON, ARCOFF, WVON, WVOFF, SEARCH_START, SEARCH_STATIC, SEARCH_DYNAMIC, SEARCH_CALC, SEARCH_END, SEARCH_OFFSET,
              SEARCH_OFFSETEND, DOUT, DIN, SETINT, SETDOUBLE, SETBOOL, SWITCHTOOL, SWITCHUSER, SWITCHSYNC};
    double m_velocity;      ///< 速度
    double m_acc;           ///< 加速度
    double m_dec;           ///< 减速度
    double m_pl;            ///< 平滑
    int m_spin;             ///< 姿态(仅MOVCA需要这个参数) 0:姿态不变 1:六轴不转 2:六轴旋转

    /**
     * @brief 点位
     *  - m_coord[0]和 m_coord[1]代表坐标系
     *  - m_coord[2]代表左右手 0-无 1-左 2-右
     *  - m_coord[3]到 m_coord[6] 给0即可
     */
    double m_coord[7];
    /**
     * @brief 点位
     *  - m_position[0]到 m_position[6]代表位置坐标，几轴机器人就写几位
     */
    double m_position[7];
    /**
     * @brief 外部轴点位
     *  - m_syncPosition[0]到 m_syncPosition[4]代表外部轴位置坐标
     */
    double m_syncPosition[5];

    /**
     * @brief 焊接参数
     */
    int m_arcParameterID;               ///< 文件标号

    /**
     * @brief 跟踪参数
     */

    //寻位开始
    int start_searchID;                 ///< 跟踪文件号
    int start_searchSenseType;          ///< 传感类型 0：线激光  1：点激光  2：电弧

    //静态寻位
    int static_searchID;                ///< 跟踪文件号
    int static_searchTableID;           ///< 寻位参数表号
    std::string static_positionOutID;   ///< 输出位置
    double static_searchTableDelay;     ///< 参数表延时

    //寻位计算
    int calc_searchArtifactType;        ///< 工件类型 0：通用 1：内外径   工件类型为内外径时，计算方式能且仅能为二维偏移 + 旋转
    int calc_searchCalculType;          ///< 计算方式 0：一维偏移  1：二维偏移  2：二维偏移 + 旋转  3：三点计算投影点  4：四点确定两条直线计算交点
                                        ///<   5：三点计算用户坐标系  6:四点计算用户坐标系  5：两点简易触碰寻位  6：三位偏移 + 旋转  7：向量计算
    std::string calc_positionOutID;     ///< 静态寻位 或 寻位计算结果输出
    std::string calc_position1ID;       ///< 寻位计算所需点1,下同
    std::string calc_position2ID;
    std::string calc_position3ID;
    std::string calc_position4ID;
    std::string calc_position5ID;
    std::string calc_position6ID;
    int calc_searchCoord;               ///< 输出用户坐标系
    double calc_length;                 ///< 计算长度（向量计算生效)

    //动态寻位
    std::string dynamic_positionID;     ///< 点位来源
    int dynamic_searchID;               ///< 文件号
    int dynamic_searchTableID;          ///< 参数表号（寻位开始点生效）
    int dynamic_searchPointEffet;       ///< 点位作用 0：准备点 1：寻位开始点
    double dynamic_searchV;             ///< 速度（准备点生效）
    int dynamic_searchPl;               ///< 平滑度（准备点生效）
    double dynamic_searchAcc;           ///< 加速度（准备点生效）
    std::string dynamic_positionOutID;  ///< 输出位置（寻位开始点生效）
    double dynamic_searchDelay;         ///< 参数表延时（寻位开始点生效）

    //寻位结束
    int end_searchID;                   ///< 跟踪文件号

    //寻位偏移
    std::string offset_positionID;      ///< 偏移量

    /**
     * @brief IO参数
     *
     *     DOUT用法示例
     *     cmd[0].cmdType = cmdPara::DOUT;
     *     cmd[0].m_groupType = 0;
     *     cmd[0].m_DOUTVariantsSource = "AUTO";
     *     cmd[0].m_portID = 1;
     *     cmd[0].m_portValue = 1;//0 或 1
     *
     */
    std::string m_DINVariantsSource;///参数形式 "INT" "BOOL" "GINT" "GBOOL" "AUTO"下同，当此参数为AUTO时，输出为手填值，其余为对应变量决定
    std::string m_DOUTVariantsSource;
    std::string m_DINVariantName;///参数形式 "GI001" "GB001" 下同
    std::string m_DOUTVariantName;
    int m_portID;/// 一路输出时为对应IO板端口号，多块IO板时按总端口数顺序编号，多路输出时为IO板数/输出路数（组号）
    int m_groupType;/// 0:一路输出 1:四路输出 2:八路输出
    int m_portValue;/// 一路输出时为 0或1 多路输出时以10进制形式储存2进制数

    /**
     * @brief 修改变量
     */

    std::string m_ValueName;///变量名
    std::string m_sourceName;///参数形式 "INT" "BOOL" "GINT" "GBOOL" "AUTO"下同，当此参数为AUTO时，输出为手填值，其余为对应变量决定
    std::string m_strValue;///参数值的字符串类型
    int m_intValue;///int型变量的参数值
    double m_doubleValue;///double型变量的参数值
    bool m_boolValue;///bool型变量的参数值

    /**
     * @brief 切换工具手、用户坐标系、外部轴
     *
     *     SWITCHUSER用法示例
     *     cmd[0].cmdType = cmdPara::SWITCHUSER用法示例;
     *     cmd[0].tool_varType = "CONST";
     *     cmd[0].tool_num = 1;
     */

    std::string tool_varType;         ///工具手参数来源 参数形式"CONST" "INT" "GINT"，当此参数为AUTO时，输出为手填值，其余为对应变量决定
    std::string tool_varName;         ///工具手参数名 参数形式 "GI001"
    int tool_num;               ///工具手编号 从1开始
    std::string user_varType;         ///用户坐标系参数来源 参数形式"CONST" "INT" "GINT"，当此参数为AUTO时，输出为手填值，其余为对应变量决定
    std::string user_varName;         ///用户坐标系参数名 参数形式 "GI001"
    int user_num;               ///用户坐标系编号 从1开始
    int sync_num;               ///外部轴编号 从1开始


    Type cmdType;           ///< 指令类型
    /**
     * @brief 指令结构体初始化
     */
    cmdPara()
    {
        m_velocity = 0;
        m_acc = 0;
        m_dec = 0;
        m_pl = 0;
        m_spin = 0;
        for(int i=0;i<7;i++)
        {
            m_coord[i] = 0;
        }
        for(int i=0;i<7;i++)
        {
            m_position[i] = 0;
        }
        for(int i=0;i<5;i++)
        {
            m_syncPosition[i] = 0;
        }
        cmdType = MOVJ;
        m_arcParameterID = 1;

        start_searchID = 0;
        start_searchSenseType = 0;

        static_searchID = 0;
        static_searchTableID = 0;
        static_positionOutID = "GP011";
        static_searchTableDelay = 0;

        calc_searchArtifactType = 0;
        calc_searchCalculType = 0;
        calc_positionOutID = "GP012";
        calc_position1ID = "GP001";
        calc_position2ID = "GP002";
        calc_position3ID = "GP003";
        calc_position4ID = "GP004";
        calc_position5ID = "GP005";
        calc_position6ID = "GP006";
        calc_searchCoord = 1;
        calc_length = 0;

        dynamic_positionID = "GP007";
        dynamic_searchID = 1;
        dynamic_searchTableID = 1;
        dynamic_searchDelay = 0;
        dynamic_searchPointEffet = 0;
        dynamic_searchV = 0;
        dynamic_searchPl = 0;
        dynamic_searchAcc = 0;
        dynamic_positionOutID = "GP013";

        end_searchID = 1;

        offset_positionID = 1;


        m_DINVariantsSource = "INT";
        m_DOUTVariantsSource = "INT";
        m_portID = 1;
        m_DINVariantName = "";
        m_DOUTVariantName = "";
        m_groupType = 0;
        m_portValue = 0;

        m_ValueName = "0";
        m_strValue = "GD001";
        m_sourceName = "";
        m_intValue = 0;
        m_doubleValue = 0;
        m_boolValue = false;

        tool_varType = "CONST";
        tool_varName = "GI001";
        tool_num = 1;
        user_varType = "CONST";
        user_varName = "GI001";
        user_num = 1;
        sync_num = 1;

    }
}cmdPara;



/**
 * @brief 连接控制器
 * @param ip 控制器ip
 */
bool connect_robot(char *ip);

/**
 * @brief 机器人上电
 */
bool servo_power_on();

/**
 * @brief 机器人下电
 */
void servo_power_off();

/**
 * @brief 关节运动
 * @param pos 点位数组，长度6
 * @param vel 速度，参数范围：0<vel≤100
 * @param coord 坐标系，参数范围：0≤coord≤3
 */
void robot_movj(double *pos,int vel,int coord,int acc = 80,int dec = 80);

/**
 * @brief 直线运动
 * @param pos 点位数组，长度6
 * @param vel 速度，参数范围：0<vel≤100
 * @param coord 坐标系，参数范围：0≤coord≤3
 */
void robot_movl(double *pos,int vel,int coord,int acc = 80,int dec = 80);

/**
 * @brief 圆弧运动
 * @param pos1 点位数组，长度6
 * @param pos2 点位数组，长度6
 * @param pos3 点位数组，长度6
 * @param vel 速度，参数范围：0<vel≤100
 * @param coord 坐标系，参数范围：0≤coord≤3
 */
void robot_movc(double *pos1,double *pos2,double *pos3,int vel,int coord,int acc = 80,int dec = 80);

/**
 * @brief 整圆运动
 * @param pos1 点位数组，长度6
 * @param pos2 点位数组，长度6
 * @param pos3 点位数组，长度6
 * @param vel 速度，参数范围：0<vel≤100
 * @param coord 坐标系，参数范围：0≤coord≤3
 */
void robot_movca(double *pos1,double *pos2,double *pos3,int vel,int coord,int acc = 80,int dec = 80);

/**
 * @brief 获取机器人运行状态
 * @param state 机器人运行状态
 *  - 0 停止
 *  - 1 暂停
 *  - 2 运行
 */
int get_robot_running_state();

/**
 * @brief 继续运行机器人
 * 只对非作业文件运行的暂停有效
 */
void continue_run();

/**
 * @brief 开始点动
 * @param axis 轴号
 * @param dir 方向
 */
void start_jogging(int axis,bool dir);

/**
 * @brief 停止点动
 * @param axis 轴号
 */
void stop_jogging(int axis);

/**
 * @brief 设置数字输出
 * @param port 端口号
 * @param value 数值,参数范围：0或1
 */
void set_dout(int port,int value);

/**
 * @brief 设置模拟输出
 * @param port 端口号
 * @param value 数值,参数范围：0≤value≤10
 */
void set_aout(int port,double value);

/**
 * @brief 查询数字输出
 * @param dout 数字输出数组
 */
void get_dout(double *dout);

/**
 * @brief 查询数字输入
 * @param din 数字输入数组
 */
void get_din(double *din);

/**
 * @brief 查询模拟输出
 * @param aout 模拟输出数组
 */
void get_aout(double *aout);

/**
 * @brief 查询模拟输入
 * @param ain 模拟输入数组
 */
void get_ain(double *ain);

/**
 * @brief 获取机器人当前位置
 * @param pos 点位数组，长度6
 */
void get_current_position(double *pos,int coord);

/**
 * @brief 获取机器人外部轴当前位置
 * @param pos 点位数组，长度5
 */
void get_current_sync_position(double *pos);

/**
 * @brief 设置点动速度
 * @param speed 速度，参数范围：0<speed≤100
 */
void set_jogging_speed(int speed);

/**
 * @brief 获取机器人当前速度
 * @return 速度值
 */
int get_jogging_speed();

/**
 * @brief 获取机器人当前模式
 * @return 当前模式 0：示教 1：远程 2：运行
 */
int get_current_mode();

/**
 * @brief 设置机器人当前模式
 * @param mode 模式 0：示教 1：远程 2：运行
 */
void set_current_mode(int mode);

/**
 * @brief 获取机器人当前坐标系
 * @return 坐标系 0：关节 1：直角 2：工具 3：用户
 */
int get_current_coord();

/**
 * @brief 设置机器人当前坐标系
 * @param coord 坐标系 0：关节 1：直角 2：工具 3：用户
 */
void set_current_coord(int coord);

/**
 * @brief 查询全局位置
 * @param posName 全局位置名 例如 "GP001"
 * @param pos 全局点位数组 长度14 前7位为机器人位置 后7位为外部轴位置
 */
void get_GPosition(char *posName,double *pos);

/**
 * @brief 设置全局位置
 * @param posName 全局位置名 例如 "GP001"
 * @param coord 坐标系 0：关节 1：直角 2：工具 3：用户
 * @param pos 机器人坐标
 * @param syncPos 机器人外部轴坐标
 */
void set_GPosition(char *posName, int coord ,double pos[7], double syncPos[7]);

/**
 * @brief 查询全局变量
 * @param varName 全局变量名 例如 "GI001"
 */
double get_global_variant(char *varName);

/**
 * @brief 设置全局变量
 * @param varName  全局变量名 例如 "GI001"
 * @param varValue 变量值
 */
void set_global_variant(char *varName,double varValue);

/**
 * @brief 查询4点标定
 * @param result
 */
void ask_four_point(double *result);

/**
 * @brief 进行4点标记
 * @param point
 * @param status
 */
void four_point_mark(int point,int status);

/**
 * @brief 4点标定计算
 * @param L1
 * @param L2
 * @param result
 */
void four_point_calculation(double L1,double L2,double *result);

/**
 * @brief 结果写入DH参数
 * @return
 */
bool result_for_DH();

/**
 * @brief 设置伺服状态
 * @param 0 停止 1 就绪
 */
void set_servo_state(int state);

/**
 * @brief 获取伺服状态
 * @return 0：停止 1：就绪 2：错误 3：运行
 */
int get_servo_state();

/**
 * @brief show_log 打印最后一条报错信息
 */
void show_log(string msg);

/**
 * @brief upload_job 上传作业文件
 * @param job 作业文件名，不带后缀，多个文件用“,”分隔
 */
void upload_job(char * job);

/**
 * @brief 下载作业文件
 */
void download_job();

/**
 * @brief delete_job 删除作业文件
 * @param job 作业文件名，不带后缀
 */
void delete_job(char *job);

/**
 * @brief open_job 打开作业文件
 * @param job 作业文件名，不带后缀
 */
void open_job(char *job);

/**
 * @brief step_job 单步运行作业文件
 * @param job 作业文件名，不带后缀
 * @param line 运行指令所在的行数
 */
void step_job(char *job,int line);

/**
 * @brief run_job 运行作业文件 需要运行模式
 * @param job 作业文件名，不带后缀
 */
void run_job(char *job);

/**
 * @brief pause_job 暂停作业文件或运动指令
 */
void pause_job();

/**
 * @brief stop_job 停止作业文件或运动指令
 */
void stop_job();

/**
 * @brief 继续运行作业文件
 * @note 需要运行模式
 */
int continue_job();

/**
 * @brief 设置机器人作业文件循环次数
 * @param num  循环次数 0为无限循环
 */
void set_runs(int num);

/**
 * @brief 连续运动模式
 * @param on 开启关闭
 *  - 0 关闭
 *  - 1 开启
 */
void continuous_motion_mode(int on);

/**
 * @brief 发送连续运动指令队列
 * @param cmd 指令数组
 * @param size 指令数组长度
 */
void send_continuous_motion_queue(cmdPara cmd[],unsigned int size);

/**
 * @brief 开始连续运动
 */
void continuous_motion_start();

/**
 * @brief 暂停连续运动
 */
void continuous_motion_suspend();

/**
 * @brief 停止连续运动
 */
void continuous_motion_stop();

/**
 * @brief 设置零点位置
 * @param axis 轴号
 * @return
 */
bool set_axis_zero_position(int axis);

/**
 * @brief 设置零点偏移
 * @param axis 需要偏移的轴数
 * @param shift 偏移量 -360° < shift < 360°
 */
void set_zero_pos_shift(int axis, double shift);

/**
 * @brief 伺服错误信息
 * @param Callback 回调函数
 */
void setErrorHandle(void (*Callback)(char*));

/**
 * @brief 伺服清错
 */
void clear_servo_error();

/**
 * @brief 切换用户坐标系
 * @param num 用户坐标系编号
 */
void set_user_coord_number(int num);

/**
 * @brief 获取单圈值
 * @param single_cycle 储存单圈值数组 长度6
 */
void get_single_cycle(double *single_cycle);

/**
 * @brief 切换工具手
 * @param tagetTool 要切换的工具手
 *  - 0 无工具手状态
 */
void switch_tool_hand(int tagetTool);

/**
 * @brief 获取当前工具手
 * @return 当前工具手编号
 */
int get_current_tool_hand();

/**
 * @brief 设置工具手参数
 * @param toolNum 要修改的工具手编号
 * @param toolparam 要修改的参数 长度6 X轴偏移量 Y轴偏移量 Z轴偏移量 绕A轴旋转量 绕B轴旋转量 绕C轴旋转量
 */
void set_tool_hand_param(int toolNum, double *toolparam);

/**
 * @brief 查询工具手参数
 * @param toolparam 工具手参数 长度6数组
 */
void query_tool_hand_param(int tagetTool,double *param);

#endif // INTERFACE_H
